@@ -1,11 +1,13 @@
+from __future__ import absolute_import
+from __future__ import print_function
 import os
 import logging
-import ConfigParser
+import six.moves.configparser
 from collections import OrderedDict
 from scipy.interpolate import interp1d
 from bisect import bisect_left
 import numpy as np
-from itertools import tee, izip
+from itertools import tee
 
 import json
 import matplotlib.pyplot as plt
@@ -19,25 +21,28 @@ from matplotlib.axes import Axes
 from astropy.io import ascii
 from astropy.io import fits
 
-import dependencies.set_shared_labels as shLab
 import sys
-import dependencies.WeightedKDE as WeightedKDE
-import dependencies.autoscale as autoscale
-from dependencies.walker_random_sampling import WalkerRandomSampling
-from dependencies import FillBetweenStep
+import pyp_beagle.dependencies.WeightedKDE 
+from pyp_beagle.dependencies.walker_random_sampling import WalkerRandomSampling
+import pyp_beagle.dependencies.autoscale
+from pyp_beagle.dependencies import FillBetweenStep
+import pyp_beagle.dependencies.set_shared_labels  as shLab
 
-from beagle_utils import BeagleDirectories, prepare_plot_saving, set_plot_ticks, plot_exists
-from beagle_filters import PhotometricFilters
-from beagle_summary_catalogue import BeagleSummaryCatalogue
+from .beagle_utils import BeagleDirectories, prepare_plot_saving, set_plot_ticks, plot_exists
+from .beagle_filters import PhotometricFilters
+from .beagle_summary_catalogue import BeagleSummaryCatalogue
 #from beagle_residual_photometry import ResidualPhotometry
-from beagle_multinest_catalogue import MultiNestCatalogue
-from beagle_posterior_predictive_checks import PosteriorPredictiveChecks
-from beagle_mock_catalogue import BeagleMockCatalogue
+from .beagle_multinest_catalogue import MultiNestCatalogue
+from .beagle_posterior_predictive_checks import PosteriorPredictiveChecks
+from .beagle_mock_catalogue import BeagleMockCatalogue
 
 # See here
 # http://peak.telecommunity.com/DevCenter/PythonEggs#accessing-package-resources
 # for an explanation on this approach to include data files
 from pkg_resources import resource_stream
+import six
+from six.moves import zip
+from six.moves import range
 
 TOKEN_SEP = ":"
 microJy = np.float32(1.E-23 * 1.E-06)
@@ -50,7 +55,7 @@ def pairwise(iterable):
     "s -> (s0,s1), (s1,s2), (s2, s3), ..."
     a, b = tee(iterable)
     next(b, None)
-    return izip(a, b)
+    return zip(a, b)
 
 class ObservedSpectrum(object):
 
@@ -64,7 +69,7 @@ class ObservedSpectrum(object):
             param_file = os.path.join(BeagleDirectories.results_dir, BeagleDirectories.param_file)
 
         if config is None:
-            config = ConfigParser.SafeConfigParser()
+            config = six.moves.configparser.SafeConfigParser()
             config.read(param_file)
 
         line = config.get('main', 'SPECTRUM FILE DESCRIPTION')
@@ -79,12 +84,12 @@ class ObservedSpectrum(object):
                 "min_rel_err" : None,  
                 }
 
-        for key, value in self.description.iteritems():
+        for key, value in six.iteritems(self.description):
 
             token = key + TOKEN_SEP
 
             if isinstance(value, dict):
-                for in_key, in_value in value.iteritems():
+                for in_key, in_value in six.iteritems(value):
                     in_token = token + in_key + TOKEN_SEP
                     if in_token in line:
                         self.description[key][in_key] = line.split(in_token)[1].split(" ")[0]
@@ -149,33 +154,23 @@ class ObservedSpectrum(object):
                 raise AttributeError(msg)
 
         # Set the redshift
+        self.data['redshift'] = None
         if self.description["redshift"]["keyword"] is not None:
-            self.data['redshift'] = np.float(hdu[1].header[self.description["redshift"]["keyword"]])
+            try:
+                self.data['redshift'] = np.float(hdu[1].header[self.description["redshift"]["keyword"]])
+            except:
+                pass
 
         # Set the mask spectrum
         if self.description["mask"]["colName"] is not None:
             self.data['mask'] = np.array(data[self.description["mask"]["colName"]], dtype=bool)
 
-
         hdu.close()
 
 class Spectrum(object):
 
     def __init__(self, params_file,
-            line_labels_json=None,
-            plot_line_labels=False,
-            resolution=None,
-            mock_catalogue=None,
-            wl_range=None,
-            wl_units='micron',
-            plot_full_SED=False,
-            print_ID=False,
-            wl_rest=False,
-            log_flux=False,
-            draw_steps=False,
-            show_residual=False,
-            plot_suffix=None,
-            n_SED_to_plot=100):
+            **kwargs):
 
         self.inset_fontsize = BeagleDirectories.inset_fontsize_fraction * BeagleDirectories.fontsize
 
@@ -183,42 +178,42 @@ class Spectrum(object):
 
         self.multinest_catalogue = MultiNestCatalogue()
 
-        self.mock_catalogue = mock_catalogue
+        self.mock_catalogue = kwargs.get('mock_catalogue')
 
         #self.residual = ResidualPhotometry()
 
         self.PPC = PosteriorPredictiveChecks()
 
-        if line_labels_json == None:
+        if kwargs.get('line_labels_json') is None:
             self.line_labels = json.load(resource_stream(__name__, 'files/emission_lines.json'), 
                     object_pairs_hook=OrderedDict)
         else:
-            with open(line_labels_json) as f:    
+            with open(kwargs.get('line_labels_json')) as f:    
                  self.line_labels = json.load(f, object_pairs_hook=OrderedDict)
 
-        self.plot_line_labels = plot_line_labels
+        self.plot_line_labels = kwargs.get('plot_line_labels', False)
 
-        self.resolution = resolution
+        self.resolution = kwargs.get('resolution')
     
-        self.wl_range = wl_range
+        self.wl_range = kwargs.get('wl_range')
 
-        self.wl_units = wl_units
+        self.wl_units = kwargs.get('wl_units', 'micron')
 
-        self.wl_rest = wl_rest
+        self.wl_rest = kwargs.get('wl_rest', False)
 
-        self.log_flux = log_flux
+        self.log_flux = kwargs.get('plot_log_flux', False)
 
-        self.plot_full_SED = plot_full_SED
+        self.plot_full_SED = kwargs.get('plot_full_SED', False)
 
-        self.show_residual = show_residual
+        self.show_residual = kwargs.get('show_residual', False)
 
-        self.print_ID = print_ID
+        self.print_ID = kwargs.get('print_ID', False)
 
-        self.draw_steps = draw_steps
+        self.draw_steps = kwargs.get('draw_steps', False)
 
-        self.n_SED_to_plot = n_SED_to_plot
+        self.n_SED_to_plot = kwargs.get('n_SED_to_plot', 100)
 
-        self.plot_suffix = plot_suffix
+        self.plot_suffix = kwargs.get('plot_suffix')
 
     def plot_marginal(self, ID, 
             observation_name=None,
@@ -351,7 +346,17 @@ class Spectrum(object):
         # Set the plot limits from the minimum and maximum wl_eff
         axs = list()
         residual_axs = list()
-        z1 = (1.+observation.data['redshift'])
+        if observation.data['redshift'] is not None:
+            redshift = observation.data['redshift']
+        else:
+            _redshifts =  hdulist['galaxy properties'].data['redshift']
+            _, _counts = np.unique(_redshifts, return_counts=True)
+            if len(_counts) > 1:
+                raise ValueError("The `redshift` of the object is not unique!")
+            redshift = _redshifts[0]
+
+        z1 = 1. + redshift
+
         if self.wl_rest:
             data_wl = observation.data['wl'] / z1
             data_flux = observation.data['flux'] * z1
@@ -384,6 +389,17 @@ class Spectrum(object):
         slices.append(np.ma.clump_unmasked(masked_array))
         slices.append(np.ma.clump_masked(masked_array))
 
+        model_mask = np.ones(len(model_wl), dtype=bool)
+        if "marginal sed mask" in hdulist:
+            model_mask = np.array(hdulist['marginal sed mask'].data['mask'][0], dtype=bool)
+
+        # Create masked versions of arrays
+        slices_model = list()
+        masked_array = np.ma.array(model_wl, mask=~model_mask)
+        slices_model.append(np.ma.clump_unmasked(masked_array))
+        slices_model.append(np.ma.clump_masked(masked_array))
+
+
         if self.show_residual:
             n_outer = 2
         else:
@@ -530,7 +546,7 @@ class Spectrum(object):
                         tick.set_rotation(45)
 
         # Define plotting styles
-        xlabel = "$\uplambda / " + xlabel + "$"
+        xlabel = "$\\uplambda / " + xlabel + "$"
         if not self.wl_rest:
             xlabel = xlabel + " (observed frame)"
 
@@ -554,10 +570,12 @@ class Spectrum(object):
             #fig.text(0.5, 0.95, str(ID).split('_')[0].strip(), ha='center', va='top')
             plt.suptitle(str(ID).split('_')[0].strip())
 
-        colors = ["red", mask_color]
+        alpha_line = 0.7
+        alpha_fill = 0.3
 
         for ax in axs:
 
+            colors = ["red", "salmon"]
             for slic, col in zip(slices, colors):
                 for s in slic:
                     if (self.draw_steps):
@@ -566,14 +584,14 @@ class Spectrum(object):
                                 where="mid",
                                 color=col,
                                 linewidth=1.50,
-                                alpha=0.7
+                                alpha=alpha_line
                                 )
                     else:
                         ax.plot(data_wl[s],
                                 data_flux[s],
                                 color=col,
                                 linewidth=2.00,
-                                alpha=0.7
+                                alpha=alpha_line
                                 )
 
                     if (self.draw_steps):
@@ -585,7 +603,7 @@ class Spectrum(object):
                                 color=col, 
                                 linewidth=0,
                                 interpolate=True,
-                                alpha=0.3
+                                alpha=alpha_fill
                                 )
                     else:
                         ax.fill_between(data_wl[s],
@@ -594,45 +612,48 @@ class Spectrum(object):
                                 facecolor=col, 
                                 linewidth=0,
                                 interpolate=True,
-                                alpha=0.3
+                                alpha=alpha_fill
                                 )
 
 
-            kwargs = { 'alpha': 0.7 }
-            if (self.draw_steps):
-                ax.step(model_wl,
-                        median_flux,
-                        where="mid",
-                        color = "blue",
-                        linewidth = 1.0,
-                        **kwargs
-                        )
-            else:
-                ax.plot(model_wl,
-                        median_flux,
-                        color = "blue",
-                        linewidth = 1.5,
-                        **kwargs
-                        )
+            colors = ["blue", "deepskyblue"]
+            for slic, col in zip(slices_model, colors):
+                for s in slic:
+                    if (self.draw_steps):
+                        ax.step(model_wl[s],
+                                median_flux[s],
+                                where="mid",
+                                color=col,
+                                linewidth = 1.0,
+                                alpha=alpha_line
+                                )
+                    else:
+                        ax.plot(model_wl[s],
+                                median_flux[s],
+                                color=col,
+                                linewidth = 1.5,
+                                alpha=alpha_line
+                                )
 
-            kwargs = { 'alpha': 0.3 }
-            if (self.draw_steps):
-                FillBetweenStep.fill_between_steps(ax,
-                        model_wl,
-                        lower_flux[:], 
-                        upper_flux[:],
-                        step_where="mid",
-                        color = "blue", 
-                        linewidth=0,
-                        **kwargs)
-            else:
-                ax.fill_between(model_wl,
-                        lower_flux[:],
-                        upper_flux[:],
-                        facecolor = "blue", 
-                        linewidth=0,
-                        interpolate=True,
-                        **kwargs)
+                    if (self.draw_steps):
+                        FillBetweenStep.fill_between_steps(ax,
+                                model_wl[s],
+                                lower_flux[s], 
+                                upper_flux[s],
+                                step_where="mid",
+                                color=col, 
+                                linewidth=0,
+                                alpha=alpha_fill
+                                )
+                    else:
+                        ax.fill_between(model_wl[s],
+                                lower_flux[s],
+                                upper_flux[s],
+                                facecolor=col, 
+                                linewidth=0,
+                                interpolate=True,
+                                alpha=alpha_fill
+                                )
 
             # Extract and plot full SED
             if 'full sed wl' in hdulist and self.plot_full_SED:
@@ -646,13 +667,11 @@ class Spectrum(object):
                     for i in rand_indices:
                         SED = hdulist['full sed'].data[i,:]
                 else:
-                    z1 = (1.+self.observed_spectrum.data['redshift'])
                     wl_obs = hdulist['full sed wl'].data['wl'][0,:] * z1
 
                     for i in rand_indices:
                         SED = hdulist['full sed'].data[i,:] / z1
 
-
                     ax.plot(wl_obs, 
                             flux_obs,
                             color="black",
@@ -660,7 +679,6 @@ class Spectrum(object):
                             lw=0.5,
                             alpha=0.5)
 
-
             kwargs = { 'alpha': 0.8 }
 
         ylim = autoscale.get_autoscale_y(axs[0], ylog=self.log_flux)
@@ -726,11 +744,11 @@ class Spectrum(object):
 
             # Label emission lines
             if self.plot_line_labels:
-                for key, label in self.line_labels.iteritems():
+                for key, label in six.iteritems(self.line_labels):
                     if self.wl_rest:
                         x = label["wl"]/wl_factor
                     else:
-                        x = label["wl"]/wl_factor * (1.+self.observed_spectrum.data['redshift'])
+                        x = label["wl"]/wl_factor * z1
 
                     if x < x0 or x > x1:
                         continue
@@ -781,31 +799,31 @@ class Spectrum(object):
 
                 # Print the evidence
                 try:
-                    ax.text(x, y, "$\log(Z)=" + "{:.2f}".format(self.logEvidence) + "$", fontsize=self.inset_fontsize)
+                    ax.text(x, y, "$\\log(Z)=" + "{:.2f}".format(self.logEvidence) + "$", fontsize=self.inset_fontsize)
                 except AttributeError:
-                    print "ciao"
+                    print("ciao")
 
                 # Print the average reduced chi-square
                 try:
                     aver_chi_square = self.PPC.data['aver_chi_square'][self.PPC.data['ID'] == ID]
                     y = y1 - (y1-y0)*0.15
-                    ax.text(x, y, "$\langle\chi^2\\rangle=" + "{:.2f}".format(aver_chi_square) + "$", fontsize=self.inset_fontsize)
+                    ax.text(x, y, "$\\langle\chi^2\\rangle=" + "{:.2f}".format(aver_chi_square) + "$", fontsize=self.inset_fontsize)
                 except AttributeError:
-                    print "`PosteriorPredictiveChecks` not computed/loaded, hence " \
-                    "<chi^2> for the object `" + str(ID) + "` is not available"
+                    print("`PosteriorPredictiveChecks` not computed/loaded, hence " \
+                    "<chi^2> for the object `" + str(ID) + "` is not available")
 
                 try:
                     aver_red_chi_square = self.PPC.data['aver_red_chi_square'][self.PPC.data['ID'] == ID]
                     n_data = self.PPC.data['n_used_bands'][self.PPC.data['ID'] == ID]
                     y = y1 - (y1-y0)*0.20
                     ax.text(x, y,
-                            "$\langle\chi^2/(\\textnormal{N}_\\textnormal{data}-1)\\rangle=" \
+                            "$\\langle\chi^2/(\\textnormal{N}_\\textnormal{data}-1)\\rangle=" \
                             + "{:.2f}".format(aver_red_chi_square) + "\; \
                             (\\textnormal{N}_\\textnormal{data}=" + \
                             "{:d}".format(n_data) + ")" + "$", fontsize=self.inset_fontsize)
                 except AttributeError:
-                    print "`PosteriorPredictiveChecks` not computed/loaded, hence " \
-                    "<chi^2_red> for the object `" + str(ID) + "` is not available"
+                    print("`PosteriorPredictiveChecks` not computed/loaded, hence " \
+                    "<chi^2_red> for the object `" + str(ID) + "` is not available")
 
             if y0 < 0.: plt.plot( [x0,x1], [0.,0.], color='gray', lw=1.0 )
 
