@@ -18,7 +18,7 @@ import six
 from six.moves import range
 from six.moves import input
 
-import pyp_beagle.dependencies.WeightedKDE 
+import pyp_beagle.dependencies.WeightedKDE as WeightedKDE
 
 ID_COLUMN_LENGTH = 100
 
@@ -99,7 +99,7 @@ def configure_matplotlib():
     # into 
     # mpl.rcParams['ytick.minor.right'] = True
 
-    mpl.rcParams['backend'] = 'TkAgg'
+    mpl.rcParams['backend'] = 'MacOSX'
 
     mpl.rc('lines', 
             linewidth=1.5, linestyle='-', 
@@ -116,8 +116,12 @@ def configure_matplotlib():
             color='black', usetex=True)
 
 
-    mpl.rcParams['text.latex.preamble'] = '\\usepackage{amssymb}, \\usepackage{amsmath}, \
-            \\usepackage{upgreek}, \\usepackage{txfonts}'
+    mpl.rcParams['text.usetex'] = True
+
+    mpl.rcParams['axes.unicode_minus'] = False
+
+    mpl.rcParams['text.latex.preamble'] = '\\usepackage{amssymb} \\usepackage{amsmath} \
+            \\usepackage{upgreek} \\usepackage{txfonts} \\usepackage[utf8]{inputenc} \\DeclareUnicodeCharacter{2212}{$-$}'
 
     mpl.rcParams['text.latex.preview'] = True
 
